import { createTheme, Palette } from "@mui/material/styles";

import nuskinTheme from "../themes/genome-foundation/";

import { FoundationTheme } from "../themes/types";

export const generatePalette = (colors?: Palette, prefersDarkMode = false) => {
    const colorKeys = Object.keys(colors || {}) as FoundationTheme.PaletteColors[];
    const paletteColors = colorKeys.reduce((result, paletteName) => {
        const colorObject = colors?.[paletteName];
        if (prefersDarkMode) {
            const darkMode = colorObject?.darkMode;
            if (darkMode) {
                result[paletteName] = darkMode;
            }
        } else if (colorObject) {
            result[paletteName] = colorObject;
        }
        return result;
    }, {} as Palette);
    return paletteColors;
};

export const createNuskinTheme = (theme: FoundationTheme.Theme = nuskinTheme, prefersDarkMode?: boolean) => {
    const muiTheme = createTheme({ ...theme });
    return {
        ...muiTheme,
        palette: {
            ...muiTheme.palette,
            ...generatePalette(theme?.palette, prefersDarkMode),
        }
    };
};

const iterateNuskinThemeColors = (
    colorObj: FoundationTheme.Colors | { [key: string]: string } = {},
) => {
    const allThemeColors: { [key: string]: string | { main?: string; dark?: string } } = {};

    const  iterate = (colorObj: FoundationTheme.Colors = {}, parentColor?: string) => {
        Object.keys(colorObj).forEach(key => {
            const colorKey = parentColor ? `${parentColor}-${key}` : key;
            const colorValue = colorObj[key as FoundationTheme.PaletteColors];
            if (colorValue?.main) {
                allThemeColors[colorKey] = colorValue as { main?: string; dark?: string };
            } else if (typeof colorValue === "object") {
                iterate(colorValue, colorKey);
            } else if (typeof colorValue === "string") {
                allThemeColors[colorKey] = colorValue;
            }
        });
    }

    iterate(colorObj);
    return allThemeColors;
}

const generateColors = (name: string, value: string | { main?: string; dark?: string }) => {
    const makeColorCss = (colorValue: string) => `
        .bg-${name} {
            background: ${colorValue};
        }
        .text-${name} {
            color: ${colorValue};
        }
        .border-${name} {
            border-color: ${colorValue};
        }
    `;
    if (typeof value !== "string" && value?.main) {
        let cssText = makeColorCss(value.main);
        if (value?.dark) {
            cssText += `
                .dark {
                    ${makeColorCss(value?.dark)}
                }
            `;
        }
        return cssText;
    }
    return makeColorCss(value as string);
};

export const generateGlobalColorCss = (theme: FoundationTheme.Theme) => {
    if (!theme) {
        return '';
    }
    
    let colorCssString = '';
    /*
     * Nuskin Theme consists of palette, which is sent to Mui
     * The Nuskin Theme also has colors which is used to generate the helper css
     */
    const palette = theme?.palette;
    
    const themeColors = iterateNuskinThemeColors(theme?.colors);
    
    const paletteColorNames = Object.keys(palette || {}) as FoundationTheme.PaletteColors[];
    
    const paletteColors = paletteColorNames?.filter((color: FoundationTheme.PaletteColors) => {
        const colorInPalette = palette?.[color];
        return colorInPalette?.main;
    });

    colorCssString = paletteColors?.reduce((result, colorName) => {
        const colorValue = palette?.[colorName]?.main;
        if (colorValue) {
            result += generateColors(colorName, colorValue);
        }
        return result;
    }, "");

    colorCssString = Object.keys(themeColors)?.reduce((result, colorName: string) => {
        const colorValue = themeColors?.[colorName];
        if (colorValue) {
            result += generateColors(colorName, colorValue);
        }
        return result;
    }, colorCssString);

    return colorCssString;
};

export const generateTextAlignCss = () => {
    return `
        .text-left {
            text-align: left;
        }
        .text-center {
            text-align: center;
        }
        .text-right {
            text-align: right;
        }
        .text-justify {
            text-align: justify;
        }
        .text-start {
            text-align: start;
        }
        .text-end {
            text-align: end;
        }
    `;
}
